@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity animation effects */
 
 /* 
@@ -23,6 +25,7 @@
 #include "effects.h"
 #include "display.h"
 #include "ui.h"
+#include "window.h"
 
 #ifdef HAVE_SHAPE
 #include <X11/extensions/shape.h>
@@ -72,6 +75,132 @@ typedef struct
   
 } BoxAnimationContext;
 
+struct MetaEffectPriv
+{
+  MetaEffectFinished finished;
+  gpointer	     finished_data;
+};
+
+static void run_default_effect_handler (MetaEffect *effect);
+static void run_handler (MetaEffect *effect);
+
+static MetaEffectHandler effect_handler;
+static gpointer		 effect_handler_data;
+
+void
+meta_push_effect_handler (MetaEffectHandler   handler,
+			  gpointer            data)
+{
+    effect_handler = handler;
+    effect_handler_data = data;
+}
+
+void
+meta_pop_effect_handler (void)
+{
+    /* FIXME: not implemented yet */
+    g_assert_not_reached ();
+}
+
+static MetaEffect *
+create_effect (MetaEffectType      type,
+	       MetaWindow         *window,
+	       MetaEffectFinished  finished,
+	       gpointer		   finished_data)
+{
+    MetaEffect *effect = g_new (MetaEffect, 1);
+
+    effect->type = type;
+    effect->window = window;
+    effect->priv = g_new (MetaEffectPriv, 1);
+    effect->priv->finished = finished;
+    effect->priv->finished_data = finished_data;
+
+    return effect;
+}
+
+void
+meta_effect_end (MetaEffect         *effect)
+{
+    if (effect->priv->finished)
+	effect->priv->finished (effect, effect->priv->finished_data);
+    
+    g_free (effect->priv);
+    g_free (effect);
+}
+
+void
+meta_effect_run_focus (MetaWindow	    *window,
+		       MetaEffectFinished   finished,
+		       gpointer		    data)
+{
+    MetaEffect *effect;
+
+    g_return_if_fail (window != NULL);
+
+    effect = create_effect (META_EFFECT_FOCUS, window, finished, data);
+    
+    run_handler (effect);
+}
+
+void
+meta_effect_run_minimize (MetaWindow         *window,
+			  MetaRectangle      *window_rect,
+			  MetaRectangle	     *icon_rect,
+			  MetaEffectFinished  finished,
+			  gpointer            data)
+{
+    MetaEffect *effect;
+
+    g_return_if_fail (window != NULL);
+    g_return_if_fail (icon_rect != NULL);
+    
+    effect = create_effect (META_EFFECT_MINIMIZE, window, finished, data);
+
+    effect->u.minimize.window_rect = *window_rect;
+    effect->u.minimize.icon_rect = *icon_rect;
+
+    run_handler (effect);
+}
+
+void
+meta_effect_run_unminimize (MetaWindow         *window,
+			    MetaRectangle      *window_rect,
+			    MetaRectangle      *icon_rect,
+			    MetaEffectFinished  finished,
+			    gpointer            data)
+{
+    MetaEffect *effect;
+
+    g_return_if_fail (window != NULL);
+    g_return_if_fail (icon_rect != NULL);
+    
+    effect = create_effect (META_EFFECT_UNMINIMIZE, window, finished, data);
+
+    effect->u.minimize.window_rect = *window_rect;
+    effect->u.minimize.icon_rect = *icon_rect;
+
+    run_handler (effect);
+}
+
+void
+meta_effect_run_close (MetaWindow         *window,
+		       MetaEffectFinished  finished,
+		       gpointer            data)
+{
+    MetaEffect *effect;
+    
+    g_return_if_fail (window != NULL);
+
+    effect = create_effect (META_EFFECT_CLOSE, window,
+			    finished, data);
+
+    run_handler (effect);
+}
+
+
+/* old ugly minimization effect */
+
 static void
 update_wireframe_window (MetaDisplay         *display,
                          Window               xwindow,
@@ -374,7 +503,7 @@ meta_effects_draw_box_animation (MetaScreen     *screen,
       attrs.override_redirect = True;
       attrs.background_pixel = BlackPixel (screen->display->xdisplay,
                                            screen->number);
-      
+
       context->wireframe_xwindow = XCreateWindow (screen->display->xdisplay,
                                                   screen->xroot,
                                                   initial_rect->x,
@@ -624,3 +753,34 @@ meta_effects_end_wireframe (MetaScreen          *screen,
   meta_ui_pop_delay_exposes (screen->ui);
 }
 
+static void
+run_default_effect_handler (MetaEffect *effect)
+{
+    switch (effect->type)
+    {
+    case META_EFFECT_MINIMIZE:
+	meta_effects_draw_box_animation (effect->window->screen,
+					 &(effect->u.minimize.window_rect),
+					 &(effect->u.minimize.icon_rect),
+					 META_MINIMIZE_ANIMATION_LENGTH,
+					 META_BOX_ANIM_SCALE);
+	break;
+
+    default:
+	break;
+    }
+}
+
+static void
+run_handler (MetaEffect *effect)
+{
+    if (effect_handler)
+    {
+	effect_handler (effect, effect_handler_data);
+    }
+    else
+    {
+	run_default_effect_handler (effect);
+	meta_effect_end (effect);
+    }
+}
